@@ -1,5 +1,5 @@
-from datetime import datetime
-from . import six
+from datetime import datetime, date, time
+import six
 try:
 	import numpy as np
 	HAS_NUMPY = True
@@ -15,6 +15,7 @@ class DataTypes(object):
 	SHARED_STRING = 5
 	STRING = 6
 	FORMULA = 7
+	EXCEL_BASE_DATE = datetime(1900, 1, 1, 0, 0, 0)
 	
 	_enumerations = ["b", "d", "e", "inlineStr", "n", "s", "str"]
 	_numberTypes = six.integer_types + (float, complex)
@@ -44,7 +45,19 @@ class DataTypes(object):
 			return DataTypes.NUMBER
 		elif HAS_NUMPY and isinstance(value, (np.floating, np.integer, np.complexfloating, np.unsignedinteger)):
 			return DataTypes.NUMBER
-		elif isinstance(value, datetime):
+		elif isinstance(value, (datetime, date, time)):
 			return DataTypes.DATE
 		else:
-			return DataTypes.ERROR
\ No newline at end of file
+			return DataTypes.ERROR
+	
+	@staticmethod
+	def to_excel_date(d):
+		if isinstance(d, datetime):
+			delta = d - DataTypes.EXCEL_BASE_DATE
+			excel_date = delta.days + (float(delta.seconds) + float(delta.microseconds) / 1E6) / (60 * 60 * 24) + 1
+			return excel_date + (excel_date > 59)
+		elif isinstance(d, date):
+			# this is why python sucks >.<
+			return DataTypes.to_excel_date(datetime(*(d.timetuple()[:6])))
+		elif isinstance(d, time):
+			return DataTypes.to_excel_date(datetime(*(DataTypes.EXCEL_BASE_DATE.timetuple()[:3]), hour=d.hour, minute=d.minute, second=d.second, microsecond=d.microsecond)) - 1
