@@ -1,13 +1,19 @@
 from . import Range
+from . import Style
+from . import Format
 from .DataTypes import DataTypes
-from . import six
+import six
 from datetime import datetime
+from xml.sax.saxutils import escape
 
 class Worksheet(object):
 	def __init__(self, name, workbook, data=None):
 		self._columns = 0 # cache this for speed
 		self._name = name
 		self._cells = {}
+		self._cell_cache = {}
+		self._styles = {}
+		self._row_styles = {}
 		self._parent = workbook
 		self._merges = [] # list of Range objects
 		self._attributes = {}
@@ -24,6 +30,10 @@ class Worksheet(object):
 			self._cells[key] = {}
 		return Range.Range((key, 1), (key, float('inf')), self) # return a row range
 
+	@property
+	def stylesheet(self):
+		return self._stylesheet
+
 	@property
 	def name(self):
 		return self._name
@@ -50,11 +60,7 @@ class Worksheet(object):
 	def range(self, start, end):
 		# convenience method
 		return Range.Range(start, end, self)
-	
-	def report_column(self, column):
-		# listener for column additions
-		self._columns = max(self._columns, column)
-	
+		
 	def add_merge(self, range):
 		for merge in self._merges:
 			if range.intersects(merge):
@@ -64,6 +70,8 @@ class Worksheet(object):
 	def get_cell_value(self, x, y):
 		if x not in self._cells:
 			self._cells[x] = {}
+		if y not in self._cells[x]:
+			return None
 		type = DataTypes.get_type(self._cells[x][y])
 		if type == DataTypes.FORMULA:
 			# remove the equals sign
@@ -76,35 +84,70 @@ class Worksheet(object):
 	def set_cell_value(self, x, y, value):
 		if x not in self._cells:
 			self._cells[x] = {}
+		if DataTypes.get_type(value) == DataTypes.DATE:
+			self.get_cell_style(x, y).format = Format.Format('yyyy-mm-dd')
 		self._cells[x][y] = value
 	
+	def get_cell_style(self, x, y):
+		if x not in self._styles:
+			self._styles[x] = {}
+		if y not in self._styles[x]:
+			self.set_cell_style(x, y, Style.Style())
+		return self._styles[x][y]
+	
+	def set_cell_style(self, x, y, value):
+		if x not in self._styles:
+			self._styles[x] = {}
+		self._styles[x][y] = value
+		self._parent.add_style(value)
+		if not self.get_cell_value(x, y):
+			self.set_cell_value(x, y, '')
+	
+	def get_row_style(self, row):
+		if row not in self._row_styles:
+			self.set_row_style(row, Style.Style())
+		return self._row_styles[row]
+		
+	def set_row_style(self, row, value):
+		self._row_styles[row] = value
+		self._parent.add_style(value)
+	
 	@property
 	def workbook(self):
 			return self._parent
 
-	_cell_cache = {}
-	def __get_cell_data(self, cell, x, y):
+	def __get_cell_data(self, cell, x, y, style):
 		if cell not in self._cell_cache:
 			type = DataTypes.get_type(cell)
+			
 			if type == DataTypes.NUMBER:
-				if cell.__class__ == int:
-					self._cell_cache[cell] = '" t="n"><v>%d</v></c>' % (cell)
-				else:
-					self._cell_cache[cell] = '" t="n"><v>%f</v></c>' % (cell)
+				self._cell_cache[cell] = '"><v>%.15g</v></c>' % (cell)
 			elif type == DataTypes.INLINE_STRING:
-				self._cell_cache[cell] = '" t="inlineStr"><is><t>%s</t></is></c>' % (cell)
+				self._cell_cache[cell] = '" t="inlineStr"><is><t>%s</t></is></c>' % escape(cell)
 			elif type == DataTypes.DATE:
-				self._cell_cache[cell] = '" t="d"><v>%s</v></c>' % (cell.strftime("%Y-%m-%dT%H:%M:%S.%f"))
+				self._cell_cache[cell] = '"><v>%s</v></c>' % (DataTypes.to_excel_date(cell))
 			elif type == DataTypes.FORMULA:
 				self._cell_cache[cell] = '"><f>%s</f></c>' % (cell)
-		# Don't cache the coordinate location
-		return '<c r="' + Range.Range.coordinate_to_string((x, y)) + self._cell_cache[cell]
-	
+		
+		if style:
+			return "<c r=\"%s\" s=\"%d%s" % (Range.Range.coordinate_to_string((x, y)), style.id, self._cell_cache[cell])
+		else:
+			return "<c r=\"%s%s" % (Range.Range.coordinate_to_string((x, y)), self._cell_cache[cell])
+			
+	def get_row_xml_string(self, row):
+		if row in self._row_styles:
+			return "<row r=\"%d\" s=\"%d\" customFormat=\"1\">" % (row, self._row_styles[row].id)
+		else:
+			return "<row r=\"%d\">" % row
+		
 	def get_xml_data(self):
-		# initialize the shared string hashtable
-		# self.shared_strings = SharedStrings.SharedStrings(self)
+		# Precondition: styles are aligned. if not, then :v
 		for x, row in six.iteritems(self._cells):
 			row_data = []
 			for y, cell in six.iteritems(self._cells[x]):
-				row_data.append(self.__get_cell_data(cell, x, y))
+				if x not in self._styles or y not in self._styles[x]:
+					style = None
+				else:
+					style = self._styles[x][y]
+				row_data.append(self.__get_cell_data(cell, x, y, style))
 			yield x, row_data
